@@ -4,7 +4,14 @@ from .Theta_nodes import ThetaW_Node, ThetaZ_Node
 from .W_nodes import W_Node, SW_Node
 from .Y_nodes import Y_Node
 from .Z_nodes import Z_Node, SZ_Node
+from .Z_nodes_GP import Z_GP_Node
+from .Z_nodes_GP_mv import Z_GP_Node_mv
 from .multiview_nodes import *
 from .basic_nodes import *
 from .nongaussian_nodes import *
 from .variational_nodes import *
+from .Sigma_node import *
+from .Kc_node import Kc_Node
+from .Kg_node import Kg_Node
+from .U_nodes import U_GP_Node_mv
+from .ZgU_node import ZgU_node
\ No newline at end of file
