@@ -16,8 +16,7 @@ from mofapy2.build_model.utils import guess_likelihoods
 from mofapy2.build_model.train_model import train_model
 from mofapy2.core import gp_utils
 
-# import matplotlib.pyplot as plt
-
+from mofapy2 import config
 
 
 def keyboardinterrupt_saver(func):
@@ -70,8 +69,10 @@ class entry_point(object):
         self.print_banner()
         self.dimensionalities = {"C":0}
         self.model = None
-        self.imputed = False # flag
-        self.Zcompleted = False # flag
+
+        # flags
+        self.imputed = False     # probabilistic imputation
+        self.Zcompleted = False  # mefisto
 
     def print_banner(self):
         """ Method to print the mofapy2 banner """
@@ -227,7 +228,10 @@ class entry_point(object):
                         data[m][p] = data[m][p].values
                     else:
                         print("Error, input data is not a numpy.ndarray or a pandas dataframe"); sys.stdout.flush(); sys.exit()
-                data[m][p] = data[m][p].astype(np.float64)
+                if config.use_float32:
+                    data[m][p] = data[m][p].astype(np.float32)
+                else:
+                    data[m][p] = data[m][p].astype(np.float64)
 
         # Save dimensionalities
         M = self.dimensionalities["M"] = len(data)
@@ -639,7 +643,7 @@ class entry_point(object):
     def set_train_options(self,
         iter=1000, startELBO=1, freqELBO=1, startSparsity=50, tolerance=None, convergence_mode="fast",
         startDrop=1, freqDrop=1, dropR2=None, nostop=False, verbose=False, quiet=False, seed=None,
-        schedule=None, gpu_mode=False, Y_ELBO_TauTrick=True, weight_views = False, 
+        schedule=None, gpu_mode=False, weight_views = False, 
         outfile=None, save_interrupted=False):
         """ Set training options """
 
@@ -648,8 +652,9 @@ class entry_point(object):
 
         self.train_opts = {}
 
-        # Maximum number of iterations
+        # Maximum and minimum number of iterations
         self.train_opts['maxiter'] = int(iter)
+        self.train_opts['min_iter'] = 0
 
         # Lower bound computation frequency
         if freqELBO is None or freqELBO==0: freqELBO=iter+1
@@ -744,11 +749,9 @@ class entry_point(object):
         self.train_opts['seed'] = int(seed)
         # s.random.seed(self.train_opts['seed'])
 
-        # Use TauTrick to speed up ELBO computation?
-        self.train_opts['Y_ELBO_TauTrick'] = Y_ELBO_TauTrick
-
         # Weight the views to avoid imbalance problems?
         self.train_opts['weight_views'] = weight_views
+        if weight_views: print("\nweight_views set to True. Weighting the ELBO (the objective function) based on the number of features per view\n")
 
         # Output file name
         self.train_opts['outfile'] = outfile
@@ -857,6 +860,11 @@ class entry_point(object):
             print("## Warping set to True: aligning the covariates across groups")
             print("##")
 
+        # Set a minium number of training iterations based on start of optimization and warping
+        self.train_opts['min_iter'] =  self.smooth_opts['start_opt']
+        if self.smooth_opts['warping']:
+            self.train_opts['min_iter'] = max(self.train_opts['min_iter'], self.smooth_opts['warping_freq']) + 1
+
        # Sparse GPs
         if sparseGP is True:
             assert not self.smooth_opts['warping'], "The warping functionality cannot be used in conjunction with the sparseGP option."
@@ -908,16 +916,15 @@ class entry_point(object):
         self.model_opts['spikeslab_weights'] = spikeslab_weights
 
         # Define whether to use group and factor-wise ARD prior for Z
-        # if ((self.dimensionalities["G"]>1) & (ard_factors==False)): 
-        #     print("WARNING: 'ard_factors' should be set to True in model_options if using multiple groups\n")
-        # always use the ARD prior on Z with more than one group except when using GPs
-        if self.dimensionalities["G"]>1: ard_factors = True
+        if ((self.dimensionalities["G"]>1) & (ard_factors==False)): 
+            print("WARNING: 'ard_factors' in model_options should be set to True if using multiple groups unless you are using MEFISTO\n")
+        # if self.dimensionalities["G"]>1: ard_factors = True
         self.model_opts['ard_factors'] = ard_factors
 
         # Define whether to use view and factor-wise ARD prior for W
-        # if ((self.dimensionalities["M"]>1) & (ard_weights==False)): 
-        #     print("WARNING: 'ard_weights' should be set to True in model_options if using multiple views\n")
-        if self.dimensionalities["M"]>1: ard_weights = True
+        if ((self.dimensionalities["M"]>1) & (ard_weights==False)): 
+            print("WARNING: 'ard_weights' in model_options should be set to True if using multiple views\n")
+        # if self.dimensionalities["M"]>1: ard_weights = True
         self.model_opts['ard_weights'] = ard_weights
 
         # Define initial number of latent factors
@@ -937,12 +944,16 @@ class entry_point(object):
           print("- View %d (%s): %s" % (m,self.data_opts["views_names"][m],self.likelihoods[m]) )
         print("\n")
 
-    def set_data_options(self, scale_views=False, scale_groups = False, center_groups = True):
+    def set_data_options(self, scale_views=False, scale_groups = False, center_groups = True, use_float32 = False):
         """ Set data processing options """
 
         if not hasattr(self, 'data_opts'): self.data_opts = {}
         self.data_opts['center_groups'] = center_groups
 
+        # change from float64 to float32 (to decrease memory usage and improve speed)
+        config.use_float32 = use_float32
+        if (use_float32): print("use_float32 set to True: replacing float64 arrays by float32 arrays to speed up computations...\n")
+
         # Scale views to unit variance
         self.data_opts['scale_views'] = scale_views
         if (scale_views): print("Scaling views to unit variance...\n")
@@ -1259,7 +1270,7 @@ def mofa(adata, groups_label: bool = None, use_raw: bool = False, use_layer: boo
          ard_weights: bool = True, ard_factors: bool = True,
          spikeslab_weights: bool = True, spikeslab_factors: bool = False,
          n_iterations: int = 1000, convergence_mode: str = "fast",
-         gpu_mode: bool = False, 
+         gpu_mode: bool = False, use_float32: bool = False,
          save_parameters: bool = False, save_data: bool = True, save_metadata: bool = True,
          seed: int = 1, outfile: Optional[str] = None,
          expectations: Optional[List[str]] = None,
@@ -1287,6 +1298,7 @@ def mofa(adata, groups_label: bool = None, use_raw: bool = False, use_layer: boo
     n_iterations (optional): upper limit on the number of iterations
     convergence_mode (optional): fast, medium, or slow convergence mode
     gpu_mode (optional): if to use GPU mode
+    use_float32 (optional): if to use float32 precision
     save_parameters (optional): if to save training parameters
     save_data (optional): if to save training data
     save_metadata (optional): if to load metadata from the AnnData object (.obs and .var tables) and save it, False by default
@@ -1296,7 +1308,7 @@ def mofa(adata, groups_label: bool = None, use_raw: bool = False, use_layer: boo
     possible expectations names include Y, W, Z, Tau, AlphaZ, AlphaW, ThetaW, ThetaZ
     outfile (optional): output file name
     save_interrupted (optional): if to save partially trained model when the training is interrupted
-    verbose (optional): print verbose information during traing
+    verbose (optional): print verbose information during training
     quiet (optional): silence messages during training procedure
     copy (optional): return a copy of AnnData instead of writing to the provided object
     """
@@ -1305,15 +1317,14 @@ def mofa(adata, groups_label: bool = None, use_raw: bool = False, use_layer: boo
 
     lik = [likelihood] if likelihood is not None else None
 
-    ent.set_data_options(scale_views=scale_views, scale_groups=scale_groups)
+    ent.set_data_options(scale_views=scale_views, scale_groups=scale_groups, use_float32=use_float32)
     ent.set_data_from_anndata(adata, groups_label=groups_label, use_raw=use_raw, use_layer=use_layer,
                               likelihoods=lik, features_subset=features_subset, save_metadata=save_metadata)
     ent.set_model_options(ard_factors=ard_factors, ard_weights=ard_weights, 
                           spikeslab_weights=spikeslab_weights, spikeslab_factors=spikeslab_factors, 
                           factors=n_factors)
     ent.set_train_options(iter=n_iterations, convergence_mode=convergence_mode, 
-                          gpu_mode=gpu_mode, Y_ELBO_TauTrick=Y_ELBO_TauTrick,
-                          seed=seed, verbose=verbose, quiet=quiet, outfile=outfile, save_interrupted=save_interrupted)
+                          gpu_mode=gpu_mode, seed=seed, verbose=verbose, quiet=quiet, outfile=outfile, save_interrupted=save_interrupted)
 
     ent.build()
     ent.run()
